/*(c) Jippy Snake 8 Developmentteam.
 *
 *
 * This code is published unter GNU Public Licence v2.
 *
 *
 */
import java.awt.*;
import java.awt.event.*;
import java.util.Set;
import javax.swing.*;

class Config extends JFrame {
	private Player p;
	private Language lang;
	private boolean goToGame;
	
	private JLabel lLang;
	private JLabel lRes;
	private JLabel lFullscreen;
	private JLabel lKeyUp;
	private JLabel lKeyDown;
	private JLabel lKeyLeft;
	private JLabel lKeyRight;
	private JLabel lKeyMenu;
	
	private JComboBox cbLang;
	private JComboBox cbRes;
	private JCheckBox cbFullscreen;
	private JButton bKeyUp;
	private JButton bKeyDown;
	private JButton bKeyLeft;
	private JButton bKeyRight;
	private JButton bKeyMenu;
	
	private JLabel lKeyInput;
	
	private JButton bExit;
	private JButton bSaveExit;
	
	private String keyUp;
	private String keyDown;
	private String keyLeft;
	private String keyRight;
	private String keyMenu;
	private String curSettingKey;
	
	public Config(Player p) {
		init(p, false);
	}
	public Config(Player p, boolean goToGame) {
		init(p, goToGame);
	}
	
	private void init(Player p, boolean goToGame) {
		this.goToGame = goToGame;
		this.p = p;
		curSettingKey = "";
		lang = new Language(p.getLanguage());
		
		lLang = new JLabel(lang.translate("LANGUAGE"));
		lRes = new JLabel(lang.translate("RESOLUTION"));
		lFullscreen = new JLabel(lang.translate("FULLSCREEN"));
		lKeyUp = new JLabel(lang.translate("KEYUP"));
		lKeyDown = new JLabel(lang.translate("KEYDOWN"));
		lKeyLeft = new JLabel(lang.translate("KEYLEFT"));
		lKeyRight = new JLabel(lang.translate("KEYRIGHT"));
		lKeyMenu = new JLabel(lang.translate("KEYMENU"));
		
		cbLang = new JComboBox();
		cbLang.setEditable(false);
		cbLang.addItem("English");
		cbLang.addItem("Deutsch");
		if (p.getLanguage().equals("en")) cbLang.setSelectedIndex(0);
		if (p.getLanguage().equals("de")) cbLang.setSelectedIndex(1);
		
		cbRes = new JComboBox();
		cbRes.setEditable(false);
		cbRes.addItem("1920 x 1080");
		cbRes.addItem("1680 x 945");
		cbRes.addItem("1440 x 810");
		cbRes.addItem("1200 x 675");
		cbRes.addItem("960 x 540");
		if (p.getScreenRes() == 1920) cbRes.setSelectedIndex(0);
		if (p.getScreenRes() == 1680) cbRes.setSelectedIndex(1);
		if (p.getScreenRes() == 1440) cbRes.setSelectedIndex(2);
		if (p.getScreenRes() == 1200) cbRes.setSelectedIndex(3);
		if (p.getScreenRes() ==  960) cbRes.setSelectedIndex(4);
		
		cbFullscreen = new JCheckBox("");
		cbFullscreen.setSelected(p.getFullscreen());
		
		keyUp = p.getKey("up");
		keyDown = p.getKey("down");
		keyLeft = p.getKey("left");
		keyRight = p.getKey("right");
		keyMenu = p.getKey("menu");
		bKeyUp = new JButton(lang.translate(keyUp));
		bKeyDown = new JButton(lang.translate(keyDown));
		bKeyLeft = new JButton(lang.translate(keyLeft));
		bKeyRight = new JButton(lang.translate(keyRight));
		bKeyMenu = new JButton(lang.translate(keyMenu));
		
		lKeyInput = new JLabel("");
		bExit = new JButton(lang.translate("EXIT"));
		bSaveExit = new JButton(lang.translate("SAVEEXIT"));
		
		setLayout(new GridLayout(10,2));
		add(lLang);
		add(cbLang);
		add(lRes);
		add(cbRes);
		add(lFullscreen);
		add(cbFullscreen);
		add(lKeyUp);
		add(bKeyUp);
		add(lKeyDown);
		add(bKeyDown);
		add(lKeyLeft);
		add(bKeyLeft);
		add(lKeyRight);
		add(bKeyRight);
		add(lKeyMenu);
		add(bKeyMenu);
		add(lKeyInput);
		add(new JLabel(""));
		if (!goToGame)
			add(bExit);
		add(bSaveExit);
		
		setTitle("Jippy Snake 8 - Config");
		
		pack();
		setVisible(true);
		
		bKeyUp.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				waitForChangeKey("up");
			}
		});
		bKeyDown.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				waitForChangeKey("down");
			}
		});
		bKeyLeft.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				waitForChangeKey("left");
			}
		});
		bKeyRight.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				waitForChangeKey("right");
			}
		});
		bKeyMenu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				waitForChangeKey("menu");
			}
		});
		lKeyInput.addKeyListener(new KeyAdapter() {
			public void keyPressed(KeyEvent e) {
				setKey(e.getKeyCode());
			}
		});
		lKeyInput.addFocusListener(new FocusAdapter() {
			public void focusLost(FocusEvent e) {
				disableSetKey();
			}
		});
		
		bSaveExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				saveExit();
			}
		});
		
		bExit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exit();
			}
		});
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				exit();
			}
		});
	}
	
	private void waitForChangeKey(String key) {
		if (!curSettingKey.equals("")) return;
		if (key.equals("up"))
			bKeyUp.setText("...");
		if (key.equals("down"))
			bKeyDown.setText("...");
		if (key.equals("left"))
			bKeyLeft.setText("...");
		if (key.equals("right"))
			bKeyRight.setText("...");
		if (key.equals("menu"))
			bKeyMenu.setText("...");
		curSettingKey = key;
		lKeyInput.requestFocus(true);
	}
	
	private void setKey(int keyCode) {
		if (curSettingKey.equals("")) return;
		String key = keyCodeToKey(keyCode);
		if (curSettingKey.equals("up")) {
			keyUp = key;
			if (conflictingKeys()) keyUp = "UNDEFINED";
		}
		if (curSettingKey.equals("down")) {
			keyDown = key;
			if (conflictingKeys()) keyDown = "UNDEFINED";
		}
		if (curSettingKey.equals("left")) {
			keyLeft = key;
			if (conflictingKeys()) keyLeft = "UNDEFINED";
		}
		if (curSettingKey.equals("right")) {
			keyRight = key;
			if (conflictingKeys()) keyRight = "UNDEFINED";
		}
		if (curSettingKey.equals("menu")) {
			keyMenu = key;
			if (conflictingKeys()) keyMenu = "UNDEFINED";
		}
		retextKeyButtons();
		curSettingKey = "";
	}
	
	private boolean conflictingKeys() {
		String cKey1 = "";
		String cKey2 = "";
		if (!keyUp.equals("UNDEFINED") && !keyDown.equals("UNDEFINED") && keyUp.equals(keyDown)) {
			cKey1 = "UP";
			cKey2 = "DOWN";
		}
		if (!keyUp.equals("UNDEFINED") && !keyLeft.equals("UNDEFINED") && keyUp.equals(keyLeft)) {
			cKey1 = "UP";
			cKey2 = "LEFT";
		}
		if (!keyUp.equals("UNDEFINED") && !keyRight.equals("UNDEFINED") && keyUp.equals(keyRight)) {
			cKey1 = "UP";
			cKey2 = "RIGHT";
		}
		if (!keyUp.equals("UNDEFINED") && !keyMenu.equals("UNDEFINED") && keyUp.equals(keyMenu)) {
			cKey1 = "UP";
			cKey2 = "MENU";
		}
		if (!keyDown.equals("UNDEFINED") && !keyLeft.equals("UNDEFINED") && keyDown.equals(keyLeft)) {
			cKey1 = "DOWN";
			cKey2 = "LEFT";
		}
		if (!keyDown.equals("UNDEFINED") && !keyRight.equals("UNDEFINED") && keyDown.equals(keyRight)) {
			cKey1 = "DOWN";
			cKey2 = "RIGHT";
		}
		if (!keyDown.equals("UNDEFINED") && !keyMenu.equals("UNDEFINED") && keyDown.equals(keyMenu)) {
			cKey1 = "DOWN";
			cKey2 = "MENU";
		}
		if (!keyLeft.equals("UNDEFINED") && !keyRight.equals("UNDEFINED") && keyLeft.equals(keyRight)) {
			cKey1 = "LEFT";
			cKey2 = "RIGHT";
		}
		if (!keyLeft.equals("UNDEFINED") && !keyMenu.equals("UNDEFINED") && keyLeft.equals(keyMenu)) {
			cKey1 = "LEFT";
			cKey2 = "MENU";
		}
		if (!keyRight.equals("UNDEFINED") && !keyMenu.equals("UNDEFINED") && keyRight.equals(keyMenu)) {
			cKey1 = "RIGHT";
			cKey2 = "MENU";
		}
		if (cKey1.equals("")) return false;
		
		try  {
			JOptionPane.showMessageDialog(this,
			                              lang.translate("SAME_KEYCODE_FOR")+": "+lang.translate(cKey1)+", "+lang.translate(cKey2)+".");
		} catch (Exception e) {
		}
		return true;
	}
	private void disableSetKey() {
		curSettingKey = "";
		retextKeyButtons();
	}
	
	private void retextKeyButtons() {
		bKeyUp.setText(lang.translate(keyUp));
		bKeyDown.setText(lang.translate(keyDown));
		bKeyLeft.setText(lang.translate(keyLeft));
		bKeyRight.setText(lang.translate(keyRight));
		bKeyMenu.setText(lang.translate(keyMenu));
	}
	
	private String keyCodeToKey(int keyCode) {
		Set<String> keySet = Globals.keyCodes.keySet();
		for (String key : keySet)
			if (Globals.keyCodes.get(key) == keyCode) return key;
		return "UNDEFINED";
	}
	
	private void saveExit() {
		if (cbLang.getSelectedIndex() == 0) p.setLanguage("en");
		if (cbLang.getSelectedIndex() == 1) p.setLanguage("de");
		
		if (cbRes.getSelectedIndex() == 0) p.setScreenRes(1920);
		if (cbRes.getSelectedIndex() == 1) p.setScreenRes(1680);
		if (cbRes.getSelectedIndex() == 2) p.setScreenRes(1440);
		if (cbRes.getSelectedIndex() == 3) p.setScreenRes(1200);
		if (cbRes.getSelectedIndex() == 4) p.setScreenRes(960);
		
		p.setFullscreen(cbFullscreen.isSelected());
		
		if (!keyUp.equals("UNDEFINED") && !keyDown.equals("UNDEFINED") &&
		    !keyLeft.equals("UNDEFINED") && !keyRight.equals("UNDEFINED") &&
		    !keyMenu.equals("UNDEFINED")) {
			p.setKey("up", keyUp);
			p.setKey("down", keyDown);
			p.setKey("left", keyLeft);
			p.setKey("right", keyRight);
			p.setKey("menu", keyMenu);
		}
		
		p.saveSavegame();
		exit();
	}
	
	private void exit() {
		if (goToGame) {
			this.dispose();
			Main.startGame(p);
		} else {
			System.exit(0);
		}
	}
} 
