/**
  * A Class to make is possible to parse and calculate complex functions.
  * Usage: First use parseFunc to parse your function.
  * Then you can use workTree to caluclate your function to any position you wish.
  * If the function could not be calculated null will be returend.
  * NOTES:
  * The parse ignores ^ bebore * or / before + or -
  * 1+2*3 will be seen as (1+2)*3
  *
  * Check your Braces!!
  * (1+2 or (1+)2 cannot be parsed.
  * Knows constants: e, pi, i
  *
  * Known functions: exp, sqrt, inv, sin, zeta
  *
  * Known links: +, -, *, /, ^, °
  * 1°2 will be seen as 1+i*2
  *
  * Known Symbols: z
  * On calculation z will be replaced with the given complex number
  *
  * Licence: GPLv2
**/
//Modified for JS8:
//--Only use for integer numbers
//--Aded < operator:
// (x<y) = 0 if y if grater than or equal to x; 1 otherwhise
//--Aded = operator:
// (x=y) = 0 if y if grater or lower than x; 1 otherwhise

import java.util.Vector;

class Functions {
	private Vector<FuncTree> fTrees = null;
	private final boolean DEBUG = false;
	
	public Functions() {
		if (DEBUG) System.out.println("NOTE: Debug-Mode in Functions.java is active!");
		fTrees = new Vector<FuncTree>();
	}
	
	//Searches and returns the functiontree with the name name
	private FuncTree getFunction(String name) {
		FuncTree ret = null;
		for (FuncTree fTree : fTrees)
			if (fTree.getName().equals(name)) ret = fTree;
		return ret;
	}
	
	/**
	  * parses a functionstring to a funtiontree and optimizesed it
	**/
	public void add(String name, String term) {
		//check, if function name is already there
		if (getFunction(name) != null) return;
		
		//if not, create a functiontree and use if for pasring
		FuncTree fTree = new FuncTree(null, true, name);
		
		//parse it
		generateTree(name, term, fTree);
		//skip unnessesary null-functions and id functions
		while (fTree.root() != null) fTree = fTree.root();
		while (fTree.getFunc().equals("id")) {
			fTree = fTree.left();
			fTree.setRoot(null, true);
		}
		//optimizesed it
		optimizeTree(fTree);
		if (DEBUG) fTree.print(0);
		fTrees.add(fTree);
	}
	
	//Generates a functiontree on given function-string func with root as the root auf the functiontree
	private void generateTree(String name, String func, FuncTree root) {
		//set work, and wRoot as root
		//wRoot means the root, we war working with (could be changed)
		FuncTree wRoot = root;
		FuncTree work = root;
		//spos means the current startpos while parsing
		int spos = 0;
		//wpos means the current workpos while pasring
		int wpos = 0;
		//if f we save the current function
		String f = "";
		
		//while not finished
		while (wpos < func.length()) {
			//on a (
			if (func.charAt(wpos) == '(') {
				//we need to save the functionname
				f = func.substring(spos, wpos);
				//if there was none, set f to id
				if (f.equals("")) f = "id";
				//set if on work and empty f
				work.setFunc(f);
				f = "";
				//find the posotion of the conurrent )
				int braceEnd = posFuncEnd(func, wpos+1);
				//generate a new left and do same with the function withing the ( and )
				FuncTree t = new FuncTree(work, true, name);
				generateTree(name, func.substring(wpos+1, braceEnd), t);
				//set wpos and spos to the next posotions
				wpos = braceEnd;
				spos = wpos+2;
			//on a link
			} else if ("+-*/<=".indexOf(func.charAt(wpos)) != -1) {
				//if no funtioname was found
				if (work.getFunc().equals("")) {
					//take the link as funktion
					work.setFunc(func.substring(spos,wpos));
				}
				//we need to change the tree: wRoot beomes the left subtree of a new root-tree
				FuncTree newRoot = new FuncTree(wRoot.root(), true, name);
				//and crate a new right-subtree, we will work with from now on
				FuncTree newWork = new FuncTree(newRoot, false, name);
				wRoot.setRoot(newRoot, true);
				//just set the new variables
				wRoot = newRoot;
				work = newWork;
				
				//set functioname of root-tree to link-symbol
				wRoot.setFunc(Character.toString(func.charAt(wpos)));
				spos = wpos+1;
			}
			wpos++;
		}
		if (work.getFunc().equals("")) {
			work.setFunc(func.substring(spos, wpos));
		}
	}
	
	//finds the position of the concurrent ) in a string.
	private int posFuncEnd(String func, int start) {
		int p = start;
		//to find the correct one, we need to count the ( in the string while searching
		int level = 0;
		if (func.charAt(p) == '(') level++;
		//while the concurrent ) was not found
		while(level >= 0) {
			//go to next pos and higher/ower level
			p++;
			if (func.charAt(p) == '(') level++;
			if (func.charAt(p) == ')') level--;
		}
		return p;
	}
	
	//optimizes the function tree
	private void optimizeTree(FuncTree t) {
		//remove "id" from left and right subtree
		while ((t.left() != null) && t.left().getFunc().equals("id")) {
			t.left().left().setRoot(t, true);
		}
		while ((t.right() != null) && t.right().getFunc().equals("id")) {
				t.right().left().setRoot(t, false);
		}
		
		//do the same with the left and right subtree
		if (t.left() != null) optimizeTree(t.left());
		if (t.right() != null) optimizeTree(t.right());
		
		//Check if the function is known
		if (("<=".indexOf(t.getFunc()) == -1) && (applyFunc(t.getFunc(), 2, 1) == 0))
			System.out.println("Unknown Symbol: "+t.getFunc());
	}
	
	/**
	  * Caluclates the with the functiontree given function on the given value z
	  * @return function(z)
	**/
	public int calc(String name, int prev, int apples, int rnd) {
		FuncTree fTree = getFunction(name);
		if (fTree == null) return 0;
		int ret = workTree(fTree, prev, apples, rnd);
		if (ret < 0) return 0;
		return ret;
	}
	
	//Private Method for working the functiontree
	private int workTree(FuncTree t, int prev, int apples, int rnd) {
		int z1 = 0;
		int z2 = 0;
		
		//Calculate eft and right subtree
		if (t.left() != null) z1 = workTree(t.left(), prev, apples, rnd);
		if (t.right() != null) z2 = workTree(t.right(), prev, apples, rnd);
		
		//replace prev or apples with the given value
		if (t.getFunc().equals("prev")) return prev;
		if (t.getFunc().equals("apples")) return apples;
		if (t.getFunc().equals("rnd")) return rnd;
		
		//Calculate the whole tree with the given values fro left and right subtree
		return applyFunc(t.getFunc(), z1, z2);
	}
	
	//apply a function on two given values. On functions, that needs only one vlue, z ist choosen for that.
	//If a function is unknown, null will be returned
	private int applyFunc(String func, int z, int w) {
		//Check, if func is a number
		int x;
		try {
			x = Integer.parseInt(func);
			//on success, return func as the real-part of a complex number
			return x;
		} catch (Exception e) {
		}
		
		//constants
		if (func.equals("prev")) return z;
		if (func.equals("apples")) return z;
		if (func.equals("rnd")) return z;
		
		//links
		//creates a complex number with the real-parts of z and w: returns Re(z)+i*Re(w)
		if (func.equals("+")) return z+w;
		if (func.equals("-")) return z-w;
		if (func.equals("*")) return z*w;
		if (func.equals("/")) {
			if (w == 0) return 0;
			return z/w;
		}
		if (func.equals("<")) {
			if (z < w) return 1;
			return 0;
		}
		if (func.equals("=")) {
			if (z == w) return 1;
			return 0;
		}
		return 0;
	}
} 
