/*(c) Jippy Snake 7/8 Developmentteam.
 *
 *
 * This code is published unter GNU Public Licence v2.
 *
 *
 */
import java.util.TimerTask;
import java.util.LinkedList;

class GameLoop extends TimerTask {
	private JS8Level l;
	private Snake s;
	private Player player;
	
	private Coord dir;
	private Coord prevDir;
	private Coord nextDir;
	private boolean movedWithNewDir;
	
	private GUIUpdater updater;
	
	private boolean appleEaten;
	private int textGr;
	private int subGr;
	private int textDir;
	private int apples;
	private boolean filled;
	private int moveDown;
	private boolean movedUp;
	
	private boolean firstMoved;
	
	private boolean campaign;
	
	private Coord zero;
	private Coord headPos;
	private Coord prevHeadPos;
	private Coord nextBodyPos;
	private Coord prevBodyPos;
	private LinkedList<Coord> newDirs;
	
	private SoundPlayer soundPlayer;
	
	private int runs;
	
	public GameLoop(GUIUpdater u, Player p) {
		s = new Snake(Coord.getZeroCoord());
		
		updater = u;
		updater.setSnake(s);
		
		player = p;
		
		dir = Coord.getZeroCoord();
		zero = Coord.getZeroCoord();
		moveDown = 4;
		prevHeadPos = null;
		nextDir = null;
		prevDir = null;
		movedWithNewDir = true;
		appleEaten = false;
		filled = false;
		apples = 0;
		newDirs = new LinkedList<Coord>();
		soundPlayer = new SoundPlayer(true);
		
		runs = 0;
		
		firstMoved = false;
	}
	
	public synchronized boolean setLevel(JS8Level lvl) {
		if (lvl == null) return false;
		if (!lvl.isValid()) return false;
		if (l != null)
			player.setLevelRecord(l.getName(), apples, false);
		Coord headPos = lvl.getStartPos();
		if (headPos == null) {
			return false;
		}
		dir = Coord.getZeroCoord();
		s.clear();
		l = lvl;
		updater.setLevel(l);
		updater.drawLevel();
		s.setHeadPos(headPos);
		updater.prepareHud();
		updater.draw(Globals.menuOpen);
		runs = 0;
		apples = 0;
		firstMoved = false;
		Globals.menuOpen = false;
		return true;
	}
	
	public void run() {
		if (l == null) return;
		if (Globals.menuOpen) return;
		
		//Get the direction for the next move
		if (!newDirs.isEmpty()) {
			soundPlayer.playSound(soundPlayer.turnSound);
			dir = newDirs.remove();
		} else {
			movedWithNewDir = true;
		}
		
		movedUp = false;
		//Remove snake mark
		if (!appleEaten)
			l.setSnake(s.getBodyPos(), false);
		if (apples == 0) l.setSnake(s.getHeadPos(), false);
		appleEaten = false;
		
		//Get previos head pos after moving
		prevHeadPos = s.getHeadPos();
		
		//Move snake
		s.move(dir);
		
		//Get headPos
		headPos = s.getHeadPos();
		
		//Move it one down, if needed and update headPos
		if ((moveDown < 4)) {
			if (!dir.equals(zero) && (headPos.getZ() > 0)) {
			if  (moveDown == getDirNum(dir)) {
				s.setHeadPos(headPos.add(new Coord(0,0,-1)));
				headPos = s.getHeadPos();
			}
			}
		}
		moveDown = 4;
		
		boolean todie = false;
		//Behaviour on level
		int tile = l.getTile(headPos,false)%6;
		if (tile == 0) {
			todie = true;
		} else if (tile == 2) /* on Ramp-Right */ {
			if (dir.isRightOf(zero)) moveUp();
			else {
				if (!dir.equals(zero))
					todie = true;
			}
		} else if (tile == 3) /* on Ramp-Down */ {
			if (dir.isDownOf(zero)) moveUp();
			else {
				if (!dir.equals(zero))
					todie = true;
			}
		} else if (tile == 4) /* on Ramp-Left */ {
			if (dir.isLeftOf(zero)) moveUp();
			else {
				if (!dir.equals(zero))
					todie = true;
			}
		} else if (tile == 5) /* on Ramp-Up */ {
			if (dir.isUpOf(zero)) moveUp();
			else {
				if (!dir.equals(zero))
					todie = true;
			}
		}
		
		int dtile = l.getTile(headPos.getLow(), false)%6;
		int ptile = l.getTile(prevHeadPos.getLow(), false)%6;
		if (dtile == 1) dtile = 0;
		if (ptile == 1) ptile = 0;
		
		if ((ptile == 0) && (dtile == 2))
			if (dir.isRightOf(zero)) {
				if (!movedUp) {
					todie = true;
				}
			} else if (dir.scaleProd(new Coord(1,0)) == 0) {
					todie = true;
			}
		if ((ptile == 0) && (dtile == 3))
			if (dir.isDownOf(zero)) {
				if (!movedUp) {
					todie = true;
				}
			} else if (dir.scaleProd(new Coord(0,1)) == 0) {
					todie = true;
			}
		if ((ptile == 0) && (dtile == 4))
			if (dir.isLeftOf(zero)) {
				if (!movedUp) {
					todie = true;
				}
			} else if (dir.scaleProd(new Coord(1,0)) == 0) {
					todie = true;
			}
		if ((ptile == 0) && (dtile == 5))
			if (dir.isUpOf(zero)) {
				if (!movedUp) {
					todie = true;
				}
			} else if (dir.scaleProd(new Coord(0,1)) == 0) {
					todie = true;
			}
		if ((ptile == 2) && (dtile == 0)) if (dir.scaleProd(new Coord(1,0)) == 0) todie = true;
		if ((ptile == 2) && (dtile == 2)) if (dir.isRightOf(zero) && !movedUp) todie = true;
		if ((ptile == 2) && (dtile == 3)) todie = true;
		if ((ptile == 2) && (dtile == 4)) if (dir.scaleProd(new Coord(1,0)) == 0) todie = true;
		if ((ptile == 2) && (dtile == 5)) todie = true;
		if ((ptile == 3) && (dtile == 0)) if (dir.scaleProd(new Coord(0,1)) == 0) todie = true;
		if ((ptile == 3) && (dtile == 2)) todie = true;
		if ((ptile == 3) && (dtile == 3)) if (dir.isDownOf(zero) && !movedUp) todie = true;
		if ((ptile == 3) && (dtile == 4)) todie = true;
		if ((ptile == 3) && (dtile == 5)) if (dir.scaleProd(new Coord(0,1)) == 0) todie = true;
		if ((ptile == 4) && (dtile == 0)) if (dir.scaleProd(new Coord(1,0)) == 0) todie = true;
		if ((ptile == 4) && (dtile == 2)) if (dir.scaleProd(new Coord(1,0)) == 0) todie = true;
		if ((ptile == 4) && (dtile == 3)) todie = true;
		if ((ptile == 4) && (dtile == 4)) if (dir.isLeftOf(zero) && !movedUp) todie = true;
		if ((ptile == 4) && (dtile == 5)) todie = true;
		if ((ptile == 5) && (dtile == 0)) if (dir.scaleProd(new Coord(0,1)) == 0) todie = true;
		if ((ptile == 5) && (dtile == 2)) todie = true;
		if ((ptile == 5) && (dtile == 3)) if (dir.scaleProd(new Coord(0,1)) == 0) todie = true;
		if ((ptile == 5) && (dtile == 4)) todie = true;
		if ((ptile == 5) && (dtile == 5)) if (dir.isUpOf(zero) && !movedUp) todie = true;
		
		if (todie) {
			soundPlayer.playSound(soundPlayer.dieSound);
			die();
			return;
		}
		if (dtile == 1) dtile = 0;
		if (dtile == 2) moveDown = 2;
		else if (dtile == 3) moveDown = 3;
		else if (dtile == 4) moveDown = 0;
		else if (dtile == 5) moveDown = 1;
		else moveDown = 4;
		
		//set Snake mark
		l.setSnake(headPos, true);
		
		//retexture Snake
			//Snake head
		textGr = getTextureGroup(headPos);
		textDir = getDirNum(dir);
		subGr = 0;
		s.setHeadTexture(20*textGr+4*subGr+textDir+1000);
		
			//Snake end
		if (apples > 0) {
			textGr = getTextureGroup(s.getBodyPos());
			subGr = 4;
			if (apples > 1)
				textDir = getDirNum(s.getNextBodyPos().sub(s.getBodyPos()));
			else if (apples == 1)
				textDir = getDirNum(s.getHeadPos().sub(s.getBodyPos()));
			s.setBodyTexture(20*textGr+4*subGr+textDir+1000);
		}
		
			//Body
		if (apples > 1) {
			textGr = getTextureGroup(prevHeadPos);
			prevBodyPos = s.getPrevPrevBodyPos();
			Coord diff1 = headPos.sub(prevHeadPos);
			Coord diff2 = prevHeadPos.sub(prevBodyPos);
			/*if (diff1.equals(diff2)) */ subGr = 2;
			if ((diff1.getX() == diff2.getY()) && (diff1.getY() == -1*diff2.getX())) subGr = 1;
			if ((diff1.getX() == -1*diff2.getY()) && (diff1.getY() == diff2.getX())) subGr = 3;
			textDir = getDirNum(diff1);
			s.setPrevBodyTexture(20*textGr+4*subGr+textDir+1000);
		}
		
		//Check for an apple
		if (l.isAppleAt(headPos)) {
			soundPlayer.playSound(soundPlayer.appleSound);
			if (!l.removeApple(headPos) && !filled) {
				player.setLevelRecord(l.getName(), apples, true);
				filled = true;
			}
			appleEaten = true;
			s.grow(headPos);
			apples++;
		}
		
		if (campaign && (apples == l.getMinApples())) {
			int nextLevel = player.getCampaignLevel()+1;
			if (nextLevel > 149) {
				nextLevel = 0;
				player.setCampaignDone(true);
			}
			player.setCampaignLevel(nextLevel);
			
			//Load next Level
			JS8Level lvl = new JS8Level(Globals.campaignLevels.elementAt(nextLevel).getFile());
			setLevel(lvl);
			dir = zero;
			if (!l.getIntroText().equals("NULL")) {
				String introText;
				if (nextLevel == 0) introText = Globals.getIntroText("finish");
				else introText = Globals.getIntroText(l.getIntroText());
				if (!introText.equals("NULL")) {
					pause();
					updater.drawIntroText(introText);
					Globals.menuOpen = true;;
				}
			}
			return;
		}
		
		if (firstMoved) l.decrTime();
		if (l.getTime() == 0) die();
		//Snake is moved and restextured, now lets draw the scene
		updater.draw(Globals.menuOpen);
		runs++;
	}
	
	private void moveUp() {
		if (headPos.getZ() < 4) {
			s.setHeadPos(headPos.add(new Coord(0,0,1)));
			headPos = s.getHeadPos();
		}
		movedUp = true;
	}
	
	//Get the needed Texturegroup for an element that is on Position c
	private int getTextureGroup(Coord c) {
		int tg = l.getTile(c.add(new Coord(0,0,-1)), false)%6;
		if (tg > 0) tg--;
		return tg;
	}
	
	//Calculates the DirectionNumber from given coordinate
	// 0 means right
	// 1 meand down
	// 2 mean left
	// 3 mean up
	private int getDirNum(Coord c) {
		if ((c.getX() == 1) && (c.getY() == 0)) return 0;
		if ((c.getX() == 0) && (c.getY() == 1)) return 1;
		if ((c.getX() == -1) && (c.getY() == 0)) return 2;
		if ((c.getX() == 0) && (c.getY() == -1)) return 3;
		return 0;
	}
	
	//If snake dies, do all the needed things here
	private synchronized void die() {
		savePlayer();
		dir = Coord.getZeroCoord();
		s.clear();
		l.reInit();
		s.setHeadPos(l.getStartPos());
		updater.prepareHud();
		updater.draw(Globals.menuOpen);
		runs = 0;
		apples = 0;
		filled = false;
		firstMoved = false;
		Globals.menuOpen = false;
		moveDown = 4;
	}
	
	public synchronized void savePlayer() {
		if (l == null) return;
		if (player == null) return;
		player.setLevelRecord(l.getName(), apples, filled);
	}
	
	//Interface to set the next moving directions
	public synchronized void setDir(Coord newDir) {
		if (Globals.menuOpen) return;
		firstMoved = true;
		if (newDirs.size() > 1) return;
		
		Coord lastDir;
		if (newDirs.size() > 0) lastDir = newDirs.getLast();
		else lastDir = dir;
		
		if (lastDir.scaleProd(newDir) == 0) newDirs.addLast(newDir);
	}
	
	public void setCampaignMode(boolean cam) {
		campaign = cam;
	}
	
	//Interface to pause/resume the game
	public synchronized void pause() {
		Globals.menuOpen = !Globals.menuOpen;
	}
	
	//Interface to chack if game is paused
	public boolean getPaused() {
		return Globals.menuOpen;
	}
}