/*(c) Jippy Snake 8 Developmentteam.
 *
 *
 * This code is published unter GNU Public Licence v2.
 *
 *
 */

import java.io.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;

public class JS8Level extends DefaultHandler {
	//Vars of the JS8Level
	private int[][][] tiles = new int[7][16][25];
	private boolean[][][] apples = new boolean[7][16][25];
	private boolean[][][] snake = new boolean[7][16][25];
	
	private Functions funcs;
	
	private String funcNewApples;
	private String funcApplesTillNew;
	private int newApples;
	private int applesTillNew;
	private int applesThisRound;
	private int numApples;
	private int rnd;
	
	private boolean blind;
	private int neededApples;
	private int time;
	private int levelTime;
	private String author;
	private String name;
	private String introText;
	private File lvlFile;
	private boolean isValid;
	
	public JS8Level(String file) {
		init(new File (file));
	}
	
	public JS8Level(File file) {
		init(file);
	}
	
	/**
	  * Initializes the Level. The Level will be read in from file
	  * @param file The Filename to read the level from
	**/
	public void init(File file) {
		for (Coord c : Coord.range(new Coord(0,0,0), new Coord(24,15,6))) {
			tiles[c.getZ()][c.getY()][c.getX()] = 0;
			apples[c.getZ()][c.getY()][c.getX()] = false;
			snake[c.getZ()][c.getY()][c.getX()] = false;
		}
		funcs = new Functions();
		funcs.add("const", "prev");
		funcNewApples = "const";
		funcApplesTillNew = "const";
		newApples = 1;
		applesTillNew = 1;
		applesThisRound = 0;
		numApples = 0;
		rnd = 0;
		blind = false;
		neededApples = 0;
		time = -1;
		levelTime = -1;
		author = "";
		name = "";
		introText = "NULL";
		lvlFile = file;
		isValid = true;
		try {
			DefaultHandler handler = this;
			SAXParser saxParser = SAXParserFactory.newInstance().newSAXParser();
			saxParser.parse(file, handler);
		} catch( Throwable t ) {
			isValid = false;
			t.printStackTrace();
		}
		
		placeApples(new Coord(-1,-1,-1));
	}
	
	//Reinitiates level
	public void reInit() {
		init(lvlFile);
	}
	
	//Methods for getting Information about level
	//Levelinfo
	public boolean isValid() {
		return isValid;
	}
	public String getName() {
		return name;
	}
	public String getAuthor() {
		return author;
	}
	public String getIntroText() {
		return introText;
	}
	public int getNewApples() {
		return newApples;
	}
	public int getApplesTillNew() {
		return applesTillNew;
	}
	public String getFuncNewApples() {
		return funcNewApples;
	}
	public String getFuncApplesTillNew() {
		return funcApplesTillNew;
	}
	public int getMinApples() {
		return neededApples;
	}
	public boolean isBlind() {
		return blind;
	}
	public int getLevelTime() {
		return levelTime;
	}
	
	//gameinfos
	/**
	  * Returns a number that represents the tile on Coord d
	  * On snake(if ignoreSnake is not set) or invalid positions 0 will be returned
	  * @param d The Coord to get the tile from
	  * @param ignoreSnake If not set 0 will be returned, if a snakepüart is on Coord d
	  * @return A number that represents the tile on Coord d
	**/
	public int getTile(Coord d, boolean ignoreSnake) {
		Coord c = d.add(new Coord(0,0,1));
		if (c.getX() < 0) return 0;
		if (c.getY() < 0) return 0;
		if (c.getX() > 24) return 0;
		if (c.getY() > 15) return 0;
		if (!ignoreSnake && getSnake(c)) return 0;
		return tiles[c.getZ()][c.getY()][c.getX()];
	}
	
	/**
	  * @return The remaining time for that Level.
	**/
	public int getTime() {
		return time;
	}
	
	/**
	  * @param d The Coord to Cehck for
	  * @return true, if an aple mis on Coord d; false otherwise
	**/
	public boolean isAppleAt(Coord d) {
		Coord c = d.add(new Coord(0,0,1));
		return apples[c.getZ()][c.getY()][c.getX()];
	}
	
	/**
	  * Checks if a part of snake is on Coord c
	  * @param c The Coord to check for
	  * @return true: if a part of snake is on Coord c; false otherwise
	**/
	private boolean getSnake(Coord c) {
		return snake[c.getZ()][c.getY()][c.getX()];
	}
	
	//Methods for changing level
	
	/**
	  * @return The remaining time for that Level.
	**/
	public void decrTime() {
		time--;
	}
	/**
	  * Removes apple on Coord d and places new if needed
	  * @return true of succes, false otherwise
	**/
	public boolean removeApple(Coord d) {
		Coord c = d.add(new Coord(0,0,1));
		apples[c.getZ()][c.getY()][c.getX()] = false;
		applesThisRound++;
		numApples++;
		if (applesThisRound == applesTillNew) {
			rnd++;
			calcNewApples();
			calcApplesTillNew();
			return placeApples(d);
		}
		return true;
	}
	
	/**
	  * Sets/Removes a Sankemark on Coord d
	  * @param d The Coord to Set/Remove for
	  * @param s Say true to set and false to remvoe mark
	**/
	public void setSnake(Coord d, boolean s) {
		if (d == null) return;
		Coord c = d.add(new Coord(0,0,1));
		if (c.getX()<0) return;
		if (c.getY()<0) return;
		if (c.getX()>24) return;
		if (c.getY()>15) return;
		snake[c.getZ()][c.getY()][c.getX()] = s;
	}
	
	/**
	  * Searches for a good startposition of snake
	  * @return a Coord, for the startpos of snake
	**/
	public Coord getStartPos() {
		int trys = 0;
		Coord c;
		c = new Coord((int) (Math.random()*25),
		              (int) (Math.random()*16),
		              (int) (Math.random()*5));
		while( (trys < 100000) && ((getTile(c, false)%6 != 1) || isAppleAt(c)) ) {
			c = new Coord((int) (Math.random()*25),
			              (int) (Math.random()*16),
			              (int) (Math.random()*5));
			trys++;
		}
		
		if (trys >= 100000) {
			System.out.println("Level '"+getName()+"' has no place for snake!");
			return null;
		}
		return c;
	}
	
	//private methods
	
	private boolean placeApples(Coord h) {
		int trys  = 0;
		Coord d;
		for(int i = 0; i < newApples; i++) {
			d = new Coord((int) (Math.random()*25),
			              (int) (Math.random()*16),
			              (int) (Math.random()*5));
			while((trys < 100000) && (isAppleAt(d) || (getTile(d, false)%6 != 1) || h.equals(d,"xyz"))) {
				d = new Coord((int) (Math.random()*25),
				              (int) (Math.random()*16),
				              (int) (Math.random()*5));
				trys++;
			}
			if (trys >= 100000) return false;
			Coord c = d.add(new Coord(0,0,1));
			apples[c.getZ()][c.getY()][c.getX()] = true;
		}
		applesThisRound = 0;
		return true;
	}
	
	//Calcuatates the new value for the numbr of apples to place next time
	private void calcNewApples() {
		newApples = funcs.calc("funcNewApples", newApples, numApples, rnd);
	}
	
	//Caluclates the next value of needed apples, unteil new apples will be placed
	private void calcApplesTillNew() {
		applesTillNew = funcs.calc("funcApplesTillNew", applesTillNew, numApples, rnd);
	}
	
	//Code for parsing levelfile
	public void startDocument() throws SAXException {
	}
	public void endDocument() throws SAXException {
	}
	public void characters( char[] buf, int offset, int len ) throws SAXException {
	}
	
	//perhaps depricated: block, door and button
	public void startElement( String namespaceURI,
	                          String localName,   // local name
	                          String qName,       // qualified name
	                          Attributes attr )
	                         throws SAXException {
		if (qName.equals("info")) {
			parseInfo(attr);
		} else if (qName.equals("func")) {
			funcs.add(attr.getValue("name"), attr.getValue("term"));
		} else if (qName.equals("quad")) {
			parseQuad(attr);
		} else if (qName.equals("cell")) {
			int f = Integer.parseInt(attr.getValue("fill"));
			int z = Integer.parseInt(attr.getValue("z"))+1;
			int y = Integer.parseInt(attr.getValue("y"));
			int x = Integer.parseInt(attr.getValue("x"));
			tiles[z][y][x] = f;
		}
	}
	
	public void endElement( String namespaceURI,
	                        String localName,     // local name
	                        String qName )        // qualified name
	                       throws SAXException {
	}
	
	private void parseInfo(Attributes attr) {
		if (attr.getValue("type").equals("author"))
			author = attr.getValue("content");
		if (attr.getValue("type").equals("name"))
			name = attr.getValue("content");
		if (attr.getValue("type").equals("startNewApples"))
			newApples = Integer.parseInt(attr.getValue("content"));
		if (attr.getValue("type").equals("funcNewApples")) {
			funcNewApples = attr.getValue("content");
			funcs.add("funcNewApples", attr.getValue("content"));
		}
		if (attr.getValue("type").equals("startApplesTillNew"))
			applesTillNew = Integer.parseInt(attr.getValue("content"));
		if (attr.getValue("type").equals("funcApplesTillNew")) {
			funcApplesTillNew = attr.getValue("content");
			funcs.add("funcApplesTillNew", attr.getValue("content"));
		}
		if (attr.getValue("type").equals("neededApples"))
			neededApples = Integer.parseInt(attr.getValue("content"));
		if (attr.getValue("type").equals("time")) {
			time = Integer.parseInt(attr.getValue("content"));
			levelTime = time;
		}
		if (attr.getValue("type").equals("blind")) {
			if (attr.getValue("content").equals("true")) blind=true;
			else blind = false;
		}
		if (attr.getValue("type").equals("introText"))
			introText = attr.getValue("content");
	}
	
	private void parseQuad(Attributes attr) {
		int f = Integer.parseInt(attr.getValue("fill"));
		for (Coord c : Coord.range( new Coord(Integer.parseInt(attr.getValue("sx")), 
		                                      Integer.parseInt(attr.getValue("sy")),
		                                      Integer.parseInt(attr.getValue("sz"))+1),
		                            new Coord(Integer.parseInt(attr.getValue("ex")),
		                                      Integer.parseInt(attr.getValue("ey")),
		                                      Integer.parseInt(attr.getValue("ez"))+1) ))
			tiles[c.getZ()][c.getY()][c.getX()] = f;
	}
	
}
