/*(c) Jippy Snake 8 Developmentteam.
 *
 *
 * This code is published unter GNU Public Licence v2.
 *
 *
 */

import java.io.*;
import java.awt.event.KeyEvent;
import javax.xml.parsers.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.*;

class Player {
	//Rootelement of xml file
	Document savegame;
	String playerFile; //path to the savegamefile
	
	public Player(String savegameFile) {
		//Create access to savegame
		try {
			playerFile = savegameFile;
			savegame = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(new File(playerFile));
		} catch( Throwable t ) {
			t.printStackTrace();
		}
	}
	
	//Methods for getting player information
	public int getLevelRecord(String levelName) {
		Element l = getElement("level", levelName);
		if (l == null) return -1;
		return Integer.parseInt(l.getAttribute("record"));
	}
	
	public boolean getLevelFilled(String levelName) {
		Element l = getElement("level", levelName);
		if (l == null) return false;
		String filled = l.getAttribute("filled");
		if (filled.equals("true")) return true;
		else return false;
	}
	
	public int getCampaignLevel() {
		Element e = getElement("info", "campaignLevel");
		if (e == null) return 0;
		return Integer.parseInt(e.getAttribute("content"));
	}
	
	public boolean getCampaignDone() {
		Element e = getElement("info", "campaignDone");
		if (e == null) return false;
		String done = e.getAttribute("content");
		if (done.equals("true")) return true;
		else return false;
	}
	
	public String getKey(String keyName) {
		Element k = getElement("control", keyName);
		if (k == null) {
			System.out.println("Player: Unkown keyname: "+keyName);
			return "";
		}
		return k.getAttribute("key");
	}
	
	public int getScreenRes() {
		Element c = getElement("config", "resolution");
		if (c == null) return 960;
		return Integer.parseInt(c.getAttribute("value"));
	}
	
	public String getLanguage() {
		Element c = getElement("config", "language");
		if (c == null) return "de";
		return c.getAttribute("value");
	}
	
	public boolean getFullscreen() {
		Element c = getElement("config", "fullscreen");
		if (c == null) return true;
		String fc = c.getAttribute("value");
		if (fc.equals("true")) return true;
		return false;
	}
	
	//Methods for modifing player information
	public void setLevelRecord(String level, int record, boolean filled) {
		boolean fill = getLevelFilled(level) || filled;
		Element l = getElement("level", level);
		//set level-filled
		l.setAttribute("filled", Boolean.toString(fill));
		
		//set record
		if (record > getLevelRecord(level))
			l.setAttribute("record", ""+record);
		
		//third: save it
		saveSavegame();
	}
	
	public void setCampaignLevel(int campaignLevel) {
		//modify in DOM
		Element cLevel = getElement("info", "campaignLevel");
		if (campaignLevel < 0) return;
		cLevel.setAttribute("content", ""+campaignLevel);
		
		saveSavegame();
	}
	
	public void setCampaignDone(boolean campaignDone) {
		Element c = getElement("info", "campaignDone");
		c.setAttribute("content", Boolean.toString(campaignDone));
		
		saveSavegame();
	}
	
	public void setKey(String key, String keyCode) {
		Element k = getElement("control", key);
		k.setAttribute("key", ""+keyCode);
	}
	
	public void setScreenRes(int res) {
		Element r = getElement("config", "resolution");
		if (res < 960) return;
		if (res > 1920) return;
		r.setAttribute("value", ""+res);
	}
	
	public void setLanguage(String lang) {
		Element r = getElement("config", "language");
		if (!lang.equals("de") && !lang.equals("en")) return;
		r.setAttribute("value", lang);
	}
	
	public void setFullscreen(boolean fc) {
		Element r = getElement("config", "fullscreen");
		if (r == null) return;
		r.setAttribute("value", ""+fc);
	}
	
	//Common Methods for getting/setting player information
	
	//searches for an Element <type name="name" ... /> and returns it.
	//If not found, it will be created.
	private Element getElement(String type, String name) {
		NodeList nl = savegame.getElementsByTagName(type);
		Element ret = null;
		
		//earch for <type name="name" ... />
		for (int i = 0; i < nl.getLength(); i++) {
			Node n = nl.item(i);
			Element e = null;
			if (n.getNodeType() == Node.ELEMENT_NODE) e = (Element) n;
			if (e == null) continue;
			if (e.getAttribute("name").equals(name)) ret = e;
		}
		if (ret != null) return ret;
		
		//if Element was not found: create it and set default values
		Element newElem = null;
		if (type.equals("control")) {
			if (name.equals("up")) {
				newElem = savegame.createElement("control");
				newElem.setAttribute("key", "UP");
			}
			if (name.equals("down")) {
				newElem = savegame.createElement("control");
				newElem.setAttribute("key", "DOWN");
			}
			if (name.equals("left")) {
				newElem = savegame.createElement("control");
				newElem.setAttribute("key", "LEFT");
			}
			if (name.equals("right")) {
				newElem = savegame.createElement("control");
				newElem.setAttribute("key", "RIGHT");
			}
			if (name.equals("menu")) {
				newElem = savegame.createElement("control");
				newElem.setAttribute("key", "ESCAPE");
			}
		}
		if (type.equals("info")) {
			if (name.equals("campaignLevel")) {
				newElem = savegame.createElement("info");
				newElem.setAttribute("content", "0");
			}
			if (name.equals("campaignDone")) {
				newElem = savegame.createElement("info");
				newElem.setAttribute("content", "false");
			}
		}
		if (type.equals("config")) {
			if (name.equals("resolution")) {
				newElem = savegame.createElement("config");
				newElem.setAttribute("value", "960");
			}
			if (name.equals("language")) {
				newElem = savegame.createElement("config");
				newElem.setAttribute("value", "de");
			}
			if (name.equals("fullscreen")) {
				newElem = savegame.createElement("config");
				newElem.setAttribute("value", "true");
			}
		}
		if (type.equals("level")) {
			newElem = savegame.createElement("level");
			newElem.setAttribute("record", "-1");
			newElem.setAttribute("filled", "false");
		}
		
		//if newElem was not createt for some reasons
		if (newElem == null) return null;
		
		//Otherwise append it and return it
		newElem.setAttribute("name", name);
		savegame.getDocumentElement().appendChild(newElem);
		savegame.normalize();
		return newElem;
	}
	
	public void saveSavegame() {
		try {
			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			DOMSource        source = new DOMSource(savegame);
			FileOutputStream os     = new FileOutputStream(new File(playerFile));
			StreamResult     result = new StreamResult(os);
			transformer.transform(source, result);
		} catch (Exception e) {
			System.out.println("On Player: ");
			e.printStackTrace();
		}
	}
} 
